import os

from sais.autotrain.dataservice.config.const import NOTIFY_AUTH_TOKEN


class EnvVarCredentialsProvider:
    def __init__(self):
        self.token = self.__get_credentials()

    def __get_credentials(self):
        bearer_token = os.getenv(NOTIFY_AUTH_TOKEN)
        # 暂时注掉，不强制认证
        # if not bearer_token:
        #     raise ValueError(f'{NOTIFY_AUTH_TOKEN} should not be null or empty.')
        return f'Bearer {bearer_token}'



from typing import Optional, List

from pydantic import BaseModel, ConfigDict, Field


class TrainResultRequest(BaseModel):
    # id: Optional[str] = None
    train_config_id: int
    model_path: Optional[str] = None
    train_start_time: str
    train_end_time: str
    train_error_msg: Optional[str] = None
    model_config = ConfigDict(
        protected_namespaces=()
    )


class InferEntity(BaseModel):
    time: str
    radiation: Optional[float] = None
    windspeed: Optional[float] = None
    power: Optional[float] = None


class InferResultRequest(BaseModel):
    infer_config_id: int
    model_id: int
    station_id: int
    infer_start_time: str
    infer_end_time: str
    infer_error_msg: Optional[str] = None
    items_len: Optional[int] = None
    items: List[InferEntity]
    model_config = ConfigDict(
        protected_namespaces=()
    )

class QueryInferRequest(BaseModel):
    query_type: int = Field(default=0, description="查询类型，0：基于场站查询，1：基于子场查询")
    ids: list[int] = Field(default=[], description="场站或子场id列表")
    area_ids: list[int] = Field(default=[], description="区域id列表")
    algos: list[str] = Field(default=[], description="算法名称列表")
    start_time: str = Field(description="开始时间, 默认:utc, 若指定其它时区，如东八区时间:2024-02-01 15:00+08或2024-02-01T15:00+08:00")
    end_time: str = Field(description="结束时间, 默认:utc, 若指定其它时区，如东八区时间:2024-02-01 15:00+08或2024-02-01T15:00+08:00")
    page: int = Field(default=1, description="页码")
    page_size: int = Field(default=500, description="每页条数")


import json
import logging
import os
import time

import requests

from sais.autotrain.dataservice.auth.auth_info import EnvVarCredentialsProvider
from sais.autotrain.dataservice.config.const import LOGGER_NAME, ENDPOINT
from loguru import logger
from sais.autotrain.dataservice.model.data_model import Task, StatusEnum, QueryNWPRequest, QueryStationRequest, \
    QueryObservationRequest, BaseRsp, BizResponse, BaseDictRsp
from sais.autotrain.dataservice.model.train_infer_model import TrainResultRequest, InferResultRequest, QueryInferRequest
from sais.autotrain.dataservice.types.biz_exception import BizException


class TrainInferHandler(object):
    def __init__(self, endpoint=ENDPOINT, auth_provider: EnvVarCredentialsProvider = None):
        self.endpoint = endpoint
        self.auth_provider = auth_provider

    def execute_train_result_submit(self, result):
        """执行训练结果提交"""
        submit_result = self.submit_train_result(result)
        return submit_result

    def execute_infer_result_submit(self, result):
        """执行推理结果提交"""
        submit_result = self.submit_infer_result(result)
        return submit_result

    def submit_train_result(self, result: TrainResultRequest):
        """提交场站结果"""
        url = f"{self.endpoint}/api/v1/train/result"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        response = requests.post(url, json=result.model_dump(by_alias=True, exclude_none=True), headers=headers)
        result_json = response.json()
        if result_json.get('success'):
            logger.info(f"Submit train result successfully.")
            return True
        else:
            logger.error(f"Failed to submit train result. Response: {result_json}")
            return False

    def submit_infer_result(self, result: InferResultRequest) -> bool:
        """提交推理结果"""
        url = f"{self.endpoint}/api/v1/infer/result"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        response = requests.post(url, json=result.model_dump(by_alias=True, exclude_none=True), headers=headers)
        result_json = response.json()
        if result_json.get('success'):
            logger.info(f"Submit infer result successfully.")
            return True
        else:
            logger.error(f"Failed to submit infer result. Response: {result_json}")
            return False

    def execute_query_infer(self, query_params):
        """查询推理结果"""
        try:
            observation_data = self.submit_query_infer(query_params)
            result = BizResponse(code=0, data=observation_data)
        except BizException as biz_err:
            result = BizResponse(code=biz_err.code, msg=biz_err.msg)
        except Exception as e:
            result = BizResponse(code=1, msg=str(e))
        return result

    def submit_query_infer(self, query: QueryInferRequest):
        """提交观测查询"""
        url = f"{self.endpoint}/api/v1/infer/query"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        response = requests.post(url, json=query.model_dump(by_alias=True, exclude_none=True), headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            return BaseDictRsp(**query_result_json.get('data'))
        else:
            logger.error(f"Failed to submit query infer. Response: {query_result_json}")
            raise ValueError(f"Failed to submit query infer.{query_result_json.get('msg')}")


from decimal import Decimal
from enum import Enum
from typing import List, Optional, Union

from pydantic import BaseModel, Field, field_validator


class Coordinate(BaseModel):
    req_lat: float
    req_lon: float


class QueryNWPRequest(BaseModel):
    """
    Query request model
    src: str - 数据源，如：ec、gfs
    start_time: str - 数据源发布起始时间，如：24010100，24年01月01号00点
    period_interval: int - 相邻两次的发布时间，如：24，单位：小时
    period: int - 从发布起始时间往后取的发布时次长度，如：365
    start_hour: int - 单次发布时间点往后取的时间点起点，如：0，单位：小时
    end_hour: int - 单次发布时间点往后取的时间点起点，如：24，单位：小时
    forecast_interval: int - 数据点的时间间隔，如：1，单位：hour
    req_lat: float - 请求的纬度，如：35.0
    req_lon: float - 请求的经度，如：109.9
    vars: List[str] - 变量列表，如：['t2m']
    levels: List[str] - 层列表，如：['100']
    workers:int - mongo方式时生效，并发查询线程数量，默认3
    """
    engine: str = "mongo"
    src: str
    start_time: str
    period_interval: int
    period: int
    start_hour: int
    end_hour: int
    forecast_interval: int
    coords: List[Coordinate]
    vars: List[str]
    levels: Optional[List[str]] = None
    workers: int = 3

    @field_validator("engine")
    def color_must_be_valid(cls, v):
        valid_engines = ["bigdata", "mongo"]
        if v not in valid_engines:
            raise ValueError("engine must be one of bigdata, mongo")
        return v

class QueryNWPBizRequest(BaseModel):
    """
    Query request model
    src: str - 数据源，如：ec/hres
    start_time: str - 数据起始时间(不一定是真正的起报时间，要逻辑计算向前获取最近的起报时间)，如：24010100，24年01月01号00点
    ndays: int - 从匹配到的最近的起报时间开始，往后n天的数据
    hours: int - 输入时刻算起的预报小时数，如24 即传入的start_time + 24小时之间的预报数据
    vars: List[str] - 变量列表，如：['t2m']
    workers:int - mongo方式时生效，并发查询线程数量，默认3
    """
    engine: str = "mongo"
    src: str
    start_time: str
    ndays: int
    hours: int
    coords: List[Coordinate]
    vars: List[str]
    workers: int = 3

class QueryAreaRequest(BaseModel):
    provinces: list[str] = Field(default=[], description="省份或区域id列表", min_items=1)
    type: int = Field(default=0, description="场站类型，0：wind，1：solar")
    start_time: str = Field(description="开始时间, 默认:utc, 若指定其它时区，如东八区时间:2024-02-01 15:00+08或2024-02-01T15:00+08:00")
    end_time: str = Field(description="结束时间, 默认:utc, 若指定其它时区，如东八区时间:2024-02-01 15:00+08或2024-02-01T15:00+08:00")
    page: int = Field(default=1, description="页码")
    page_size: int = Field(default=500, description="每页条数")

class QueryStationRequest(BaseModel):
    query_type: int = Field(default=0, description="查询类型，0：基于场站查询，1：基于子场查询 ")
    ids: list[int] = Field(default=[], description="场站或子场id列表")
    provinces: list[str] = Field(default=[], description="省份列表")
    page: int = Field(default=1, description="页码")
    page_size: int = Field(default=500, description="每页条数")


class QueryObservationRequest(BaseModel):
    query_type: int = Field(default=0, description="查询类型，0：基于场站查询，1：基于子场查询")
    ids: list[int] = Field(default=[], description="场站或子场id列表")
    provinces: list[str] = Field(default=[], description="省份或区域id列表")
    type: int = Field(default=None, description="场站类型，0：wind，1：solar")
    start_time: str = Field(description="开始时间, 默认:utc, 若指定其它时区，如东八区时间:2024-02-01 15:00+08或2024-02-01T15:00+08:00")
    end_time: str = Field(description="结束时间, 默认:utc, 若指定其它时区，如东八区时间:2024-02-01 15:00+08或2024-02-01T15:00+08:00")
    page: int = Field(default=1, description="页码")
    page_size: int = Field(default=500, description="每页条数")


class StatusEnum(str, Enum):
    PENDING = "PENDING"
    RUNNING = "RUNNING"
    FINISHED = "FINISHED"
    FAILED = "FAILED"


class Task(BaseModel):
    task_id: str
    status: StatusEnum
    file_path: Optional[str] = None
    start_time: Optional[str] = None  # YYYY-MM-DD HH:MM:SS

class AreasDto(BaseModel):
    area_id: Optional[str] = None
    province: Optional[str] = None
    types: Optional[int] = None
    capacity: Optional[float] = None
    time: Optional[str] = None

class StationsDto(BaseModel):
    id: int = Field(default=None, description="场站id")
    code: Optional[str] = None
    types: str
    capacity: Optional[float] = None
    province: Optional[str] = None
    longitude: Decimal
    latitude: Decimal
    altitude: Optional[float] = None
    hub_height: Optional[float] = None


class ObservationsDto(BaseModel):
    time: str
    station_id: int
    sub_station_id: Optional[int] = None
    radiation: Optional[float] = None
    windspeed: Optional[float] = None
    power: Optional[float] = None
    label_status: Optional[int] = None


class BaseRsp(BaseModel):
    page: int = 0
    page_size: int = 0
    records: Union[List[StationsDto], List[ObservationsDto], List[AreasDto], None] = None


class BaseDictRsp(BaseModel):
    page: int = 0
    page_size: int = 0
    records: object = None

class BizResponse(BaseModel):
    code: int = 0  # 0: success !0: fail
    msg: str = 'ok'
    data: object = None



import json
import sys
from sais.autotrain.dataservice import EnvVarCredentialsProvider, TrainInferHandler, TrainResultRequest, \
    InferResultRequest, QueryNWPHandler, ENDPOINT, QueryNWPRequest, QueryNWPBizRequest, QueryStationRelatedHandler, QueryStationRequest, \
    QueryObservationRequest, QueryInferRequest, QueryAreaRequest
import pandas as pd




query_station_result = QueryStationRelatedHandler(endpoint="http://fuxi-ai.cn:10115", auth_provider=EnvVarCredentialsProvider())
# result = query_related.execute_query_area(QueryAreaRequest(
#     type=0,  # 0: wind, 1: solar
#     provinces=['Hainan'],
#     start_time='2024-01-31 16:00:00',
#     end_time='2024-12-10 15:00:00',
#     page=1,
#     page_size=sys.maxsize
# ))
# print(f"Query area result: {result}")

# query_station_result = query_station_result.execute_query_station(QueryStationRequest(
#     ids=["4"],
#     # provinces=["Hainan"],
#     page=1,
#     page_size=100,
#     query_type=0
# ))
# print(f"Query station result: {query_station_result.model_dump_json()}")

# result = query_related.execute_query_observation(QueryObservationRequest(
#     ids=["18"],
#     # provinces=["Hainan"],
#     start_time="2018-02-01 15:00:00",
#     end_time="2024-12-01 15:00:00",
#     page=1,
#     page_size=10,
#     query_type=0
# ))
# print(f"Query observation result: {result.model_dump_json()}")


# 全页遍历
# page = 1
# page_size = 1000
# while True:
#     query_observation_result_item = query_station_related.execute_query_observation(QueryObservationRequest(
#         ids=["1"],
#         # provinces=["山东"],
#         start_time="2023-01-01 00:00:05",
#         end_time="2024-07-01 00:00:05",
#         page=page,
#         page_size=page_size,
#         query_type=0
#     ))
#     if len(query_observation_result_item.records) == 0:
#         break
#     else:
#         print(f"Query observation result: {query_observation_result_item}")
#     page += 1

# query_nwp = QueryNWPHandler(endpoint=ENDPOINT, auth_provider=EnvVarCredentialsProvider())
# result = query_nwp.execute_query(query_params=QueryNWPRequest(
#     engine="mongo",
#     src="ec/hres_irr",
#     start_time="23080100",
#     period_interval=24,
#     period=1,
#     start_hour=28,
#     end_hour=51,
#     forecast_interval=1,
#     coords=[{"req_lat": 18.01, "req_lon": 96.61},{"req_lat": 18, "req_lon": 114},{"req_lat": 19, "req_lon": 119}],
#     vars=[
#         "ssrgrd"
#     ]))
# print(f"Query nwp result: {result.model_dump_json()}")





# query_nwp = QueryNWPHandler(endpoint=ENDPOINT, auth_provider=EnvVarCredentialsProvider())
# result = query_nwp.execute_query(query_params=QueryNWPRequest(
#     engine="mongo",
#     src="ec/hres",
#     start_time="24083000",
#     period_interval=24,
#     period=1,
#     start_hour=40,
#     end_hour=51,
#     forecast_interval=1,
#     coords=[{"req_lat": 38.44889, "req_lon": 111.4655}],
#     vars=[
#         "u100"
#     ]))
# print(f"Query nwp result: {result.model_dump_json()}")


# query_nwp = QueryNWPHandler(endpoint=ENDPOINT, auth_provider=EnvVarCredentialsProvider())
# result = query_nwp.query_nwp_meta(src="ec/hres")
# print(f"Query nwp meta: {result.model_dump_json()}")


# train_infer_handler = TrainInferHandler(endpoint="http://localhost:10115", auth_provider=EnvVarCredentialsProvider())
# train_result = train_infer_handler.execute_train_result_submit(TrainResultRequest(
#     train_config_id=62,
#     model_path="/path1/path2/model2.pkl",
#     train_start_time="2024-09-10 10:00:00",
#     train_end_time="2024-09-10 10:10:00",
#     train_error_msg="success"
# ))
# print(f"submit train result: {train_result}")

# infer_result = train_infer_handler.execute_infer_result_submit(InferResultRequest(
#     infer_config_id=1,
#     model_id=1,
#     station_id=1,
#     infer_start_time="2024-09-10 10:00:00",
#     infer_end_time="2024-09-10 10:10:00",
#     infer_status=2,  # 2：成功 3：失败
#     infer_error_msg="success",
#     items=[
#         {
#             "time": "2024-09-10 10:00:00",
#             "station_id": 1,
#             "radiation": 11.1,
#             "power": 9.8,
#         }
#     ]
# ))
# print(f"submit infer result: {infer_result}")


# train_infer_handler = TrainInferHandler(endpoint="http://fuxi-ai.cn:10115", auth_provider=EnvVarCredentialsProvider())
# query_infer_result = train_infer_handler.execute_query_infer(QueryInferRequest(
#     ids=[4],
#     algos=["xgb_wind_ultra_points"],
#     start_time="2024-03-03 00:00:00",
#     end_time="2024-03-10 00:00:00",
#     page=1,
#     page_size=9223372036854775807
# ))
# dicts = query_infer_result.model_dump()
# print(f"Query infer result: {dicts}")



# query_nwp = QueryNWPHandler(endpoint=ENDPOINT, auth_provider=EnvVarCredentialsProvider())
# result = query_nwp.execute_query(query_params=QueryNWPRequest(
#     engine="mongo",
#     src="ec/hres_irr",
#     start_time="24080100",
#     period_interval=24,
#     period=1,
#     start_hour=0,
#     end_hour=245,
#     forecast_interval=1,
#     coords=[{"req_lat": 18, "req_lon": 96.6}],
#     vars=[
#         "ssrgrd"
#     ]))
# print(f"Query nwp result: {result.model_dump_json()}")

# query_nwp = QueryNWPHandler(endpoint=ENDPOINT, auth_provider=EnvVarCredentialsProvider())
# result = query_nwp.execute_query(query_params=QueryNWPBizRequest(
#         src='ec/hres',
#         start_time='23092116',
#         ndays=160,
#         hours=24,
#         coords=[{"req_lat": 35.56544828, "req_lon": 113.21735880}],
#         vars=['u100', 'v100', 'u10', 'v10', 'msl', 't2m', 'tp']))
# print(f"Query nwp result: {result.model_dump_json()}")

#
# query_nwp = QueryNWPHandler(endpoint=ENDPOINT, auth_provider=EnvVarCredentialsProvider())
# result = query_nwp.execute_query(query_params=QueryNWPBizRequest(
#     src="ec/hres_irr",
#     start_time="24082816",
#     ndays=1,
#     hours=24,
#     coords=[{"req_lat": 19, "req_lon": 109}],
#     vars=[
#         "ssrgrd"
#     ]))
# print(f"Query nwp result: {result.model_dump_json()}")


# query_nwp = QueryNWPHandler(endpoint=ENDPOINT, auth_provider=EnvVarCredentialsProvider())
# result = query_nwp.execute_query(query_params=QueryNWPBizRequest(
#     src="ec/hres_irr",
#     start_time="23090101",
#     ndays=4,
#     hours=24,
#     coords=[{"req_lat": 18.01, "req_lon": 96.61},{"req_lat": 18, "req_lon": 114},{"req_lat": 19, "req_lon": 119}],
#     vars=[
#         "ssrgrd"
#     ]))
# print(f"Query nwp result: {result.model_dump_json()}")

# query_nwp = QueryNWPHandler(endpoint=ENDPOINT, auth_provider=EnvVarCredentialsProvider())
# result = query_nwp.execute_query(query_params=QueryNWPBizRequest(
#     src="fuxi_irr",
#     start_time="23090101",
#     ndays=4,
#     hours=24,
#     coords=[{"req_lat": 18, "req_lon": 96}],
#     vars=[
#         "ssrgrd"
#     ]))
# print(f"Query nwp result: {result.model_dump_json()}")


# 定义数据
# data = {
#     'time': ['2023-01-01 00:00:00+08'], # 带时区的字符串
#     'station_id': [18],
#     'radiation': [0.0],
#     'windspeed': [None],
#     'power': [0.02],
#     'label_status': [10]
# }
# df = pd.DataFrame(data)
#
# update_obs_related = QueryStationRelatedHandler(endpoint="http://fuxi-ai.cn:10115", auth_provider=EnvVarCredentialsProvider())
# update_obs_result = update_obs_related.execute_label_observation(df)
# print(update_obs_result)


# query_nwp = QueryNWPHandler(endpoint=ENDPOINT, mongo_host="fuxi-ai.cn", mongo_port=28087, mongo_user="root", mongo_password="yourPassword")
# result = query_nwp.execute_query(query_params=QueryNWPBizRequest(
#     src="fuxi_irr",
#     start_time="23090101",
#     ndays=4,
#     hours=24,
#     coords=[{"req_lat": 18, "req_lon": 96}],
#     vars=[
#         "ssrgrd"
#     ]))
# print(f"Query nwp result: {result.model_dump_json()}")


query_nwp = QueryNWPHandler(endpoint=ENDPOINT, mongo_host="fuxi-ai.cn", mongo_port=28087, mongo_user="root", mongo_password="yourPassword")
# result = query_nwp.execute_query(query_params=QueryNWPBizRequest(
#     src="fuxi_irr",
#     start_time="23090101",
#     ndays=4,
#     hours=24,
#     coords=[{"req_lat": 18, "req_lon": 96}],
#     vars=[
#         "ssrgrd"
#     ]))
# print(f"Query nwp result: {result.model_dump_json()}")

# result = query_nwp.execute_query(query_params=QueryNWPBizRequest(
#         src='ec/hres',
#         start_time='23070100',
#         ndays=100,
#         hours=24,
#         coords=[{'req_lat': 18, 'req_lon': 96}],
#         # coords=[{'req_lat': 18.66933828300006, 'req_lon': 109.11524498800003}, {'req_lat': 18.66933828300006, 'req_lon': 109.61524498800003}, {'req_lat': 18.66933828300006, 'req_lon': 110.11524498800003}, {'req_lat': 19.16933828300006, 'req_lon': 109.11524498800003}, {'req_lat': 19.16933828300006, 'req_lon': 109.61524498800003}, {'req_lat': 19.16933828300006, 'req_lon': 110.11524498800003}, {'req_lat': 19.66933828300006, 'req_lon': 109.61524498800003}, {'req_lat': 19.66933828300006, 'req_lon': 110.11524498800003}, {'req_lat': 19.66933828300006, 'req_lon': 110.61524498800003}],
#         vars=['u100', 'v100', 'u10', 'v10', 'msl', 't2m', 'tp']))
#         # vars=['u100m', 'v100m', 'u10m', 'v10m', 'msl', 't2m', 'tp']))
# print(f"Query nwp result: {result.model_dump_json()}")


result = query_nwp.execute_query(query_params=QueryNWPBizRequest(
        src='ec/hres',
        start_time='24083112',
        ndays=2,
        hours=24,
        coords=[{'req_lat': 18.66933828300006, 'req_lon': 109.11524498800003}],
        vars=['d2m', 't2m', 'hcc', 'mcc', 'lcc', 'tp', 'ssrd']))


print(result)


import redis
from loguru import logger
import time

from sais.autotrain.dataservice.cfg.env import CustomEnv


class RedisClient:
    def __init__(self, host=None, port=None, db=None, password=None):
        self.host = host or CustomEnv.get_redis_host()
        self.password = password or CustomEnv.get_redis_password()
        self.port = port or CustomEnv.get_redis_port()
        self.db = db or CustomEnv.get_redis_db()
        self.pool = None
        self.r = None
        self.connect()

    def connect(self):
        try:
            logger.info("Connecting to Redis...")
            self.pool = redis.ConnectionPool(host=self.host, port=self.port, db=self.db, password=self.password)
            self.r = redis.Redis(connection_pool=self.pool)
            r_res = self.r.ping()
            if not r_res:
                logger.error("Failed to connect to Redis, exiting with code 1")
                exit(1)
            logger.info("Connected to Redis")
        except Exception as e:
            logger.error(f"Failed to connect to Redis, error={e}")
            self.reconnect()

    def reconnect(self):
        logger.info("Attempting to reconnect to Redis...")
        time.sleep(5)  # Wait before reconnecting
        self.connect()

    def hgetall(self, key):
        try:
            return self.r.hgetall(key)
        except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
            logger.error(f"Redis connection error: {e}")
            self.reconnect()
            return self.hgetall(key)

    def hset(self, key, field, value):
        try:
            return self.r.hset(key, field, value)
        except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
            logger.error(f"Redis connection error: {e}")
            self.reconnect()
            return self.hset(key, field, value)

    def zrangebyscore(self, key, min_score, max_score):
        try:
            return self.r.zrangebyscore(key, min_score, max_score)
        except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
            logger.error(f"Redis connection error: {e}")
            self.reconnect()
            return self.zrangebyscore(key, min_score, max_score)

    def zadd(self, key, score, value):
        try:
            return self.r.zadd(key, {value: score})
        except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
            logger.error(f"Redis connection error: {e}")
            self.reconnect()
            return self.zadd(key, score, value)

    def zdel(self, key, score, value):
        try:
            return self.r.zrem(key, {value: score})
        except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
            logger.error(f"Redis connection error: {e}")
            self.reconnect()
            return self.zadd(key, score, value)


    def zremrangebyscore(self, key, min, max):
        try:
            return self.r.zremrangebyscore(key, min, max)
        except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
            logger.error(f"Redis connection error: {e}")
            self.reconnect()
            return self.zremrangebyscore(key, min, max)

    def zupdatebyscore(self, key, score, value):
        try:
            self.zremrangebyscore(key, score, score)
            self.zadd(key, score, value)
        except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
            logger.error(f"Redis connection error: {e}")
            self.reconnect()
            return self.zupdatebyscore(key, score, value)


    def zcard(self, key):
        try:
            return self.r.zcard(key)
        except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
            logger.error(f"Redis connection error: {e}")
            self.reconnect()
            return self.zcard(key)

    def set(self, key, value, expire_sec):
        try:
            return self.r.set(key, value, expire_sec)  # Set the key with an expiration time of 7 days
        except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
            logger.error(f"Redis connection error: {e}")
            self.reconnect()
            return self.set(key, value, expire_sec)

    def delete(self, key):
        try:
            return self.r.delete(key)  # Set the key with an expiration time of 7 days
        except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
            logger.error(f"Redis connection error: {e}")
            self.reconnect()
            return self.delete(key)


# Instantiate the Redis client
# redis_client = RedisClient()


import os
from pymongo import MongoClient
class MongoConfig:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(MongoConfig, cls).__new__(cls)
            cls._instance._initialize_from_env()
        return cls._instance

    def _initialize_from_env(self):
        self.host = os.getenv('MONGO_HOST', 'localhost')
        self.port = int(os.getenv('MONGO_PORT', 8087))
        self.user = os.getenv('MONGO_USER', "root")
        self.password = os.getenv('MONGO_PASSWORD', "yourPassword")
        self.__set_client()

    def set_config(self, host=None, port=None, user=None, password=None):
        if host:
            self.host = host
        if port:
            self.port = int(port)
        if user:
            self.user = user
        if password:
            self.password = password
        self.__set_client()
    def __set_client(self):
        mongo_url = f'mongodb://{self.user}:{self.password}@{self.host}:{self.port}'
        client = MongoClient(mongo_url,
                             serverSelectionTimeoutMS=30000,
                             connectTimeoutMS=30000)
        self.client = client

    def get_client(self):
        return self.client

    def close(self):
        if self.client:
            self.client.close()
            self.client = None


mongo_config = MongoConfig()


import concurrent
import json
import os
import re
import time
from concurrent.futures import ThreadPoolExecutor
from datetime import timedelta
import numpy as np

import pandas as pd
from dateutil.relativedelta import relativedelta
from loguru import logger
from pymongo import MongoClient

from sais.autotrain.dataservice.config.const import DEBUG
from sais.autotrain.dataservice.config.mongo_config import mongo_config
from sais.autotrain.dataservice.model.data_model import Coordinate
from sais.autotrain.dataservice.types.biz_exception import BizException
from bson.objectid import ObjectId

def query_nwp_mongo_biz(src: str, start_time: str, ndays: int, hours: int, coords: list[Coordinate], vars: list[str],
                        workers: int):
    nwp_meta = query_nwp_meta_info(src)
    # start_time = get_max_start_time(start_time, nwp_meta['start_time'])
    rule_start_time = get_rule_start_time(start_time)
    nearest_start_time = get_nearest_start_time(rule_start_time, nwp_meta['times'].split(','))
    real_start_time = get_real_start_time(nearest_start_time, nwp_meta.get('latest_time'), ndays)

    period_interval = 24
    period = ndays
    start_hour = calculate_hours_between(real_start_time, start_time)
    end_hour = (start_hour + hours) - 1
    # # 对于没有step0的属于源，需要特殊处理
    # if 'P0DT0H0M0S' not in json.loads(nwp_meta['steps']) and start_hour == 0:
    #     start_hour = 1
    #     end_hour = end_hour + 1
    forecast_interval = 1
    return query_nwp_mongo(src, real_start_time, period_interval, period, start_hour, end_hour,
                           forecast_interval, coords, vars, workers, nwp_meta)


def query_nwp_mongo(src: str, start_time: str, period_interval: int, period: int, start_hour: int, end_hour: int,
                    forecast_interval: int, coords: list[Coordinate], vars: list[str], workers: int,
                    nwp_meta: dict = None):
    if DEBUG:
        logger.debug(
            f'params: src: {src}, start_time: {start_time}, period_interval: {period_interval}, period: {period}, start_hour: {start_hour}, '
            f'end_hour: {end_hour}, forecast_interval: {forecast_interval}, coords: {coords}, vars: {vars}, workers: {workers}')
    # 记录查询开始时间
    query_start_time = time.time()
    # 查询气象源元信息
    if not nwp_meta:
        nwp_meta = query_nwp_meta_info(src)

    # 解析请求参数
    start_time_dt = datetime.strptime(start_time, "%y%m%d%H")
    # 所有起报时间点
    forcast_start_times = []
    # 所有预报时间点
    forcast_times = []
    # 预报步长
    steps = []
    # forcast_time 按月分组
    month_groups = {}
    # 发布时次
    for i in range(period):
        forcast_start_time = start_time_dt + timedelta(hours=i * period_interval)
        # logger.info(f'forcast_start_time: {forcast_start_time}')
        forcast_start_time_str = forcast_start_time.strftime("%Y-%m-%d %H:%M:%S")
        forcast_start_times.append(forcast_start_time_str)
        # 记录到月分组中
        month_key = forcast_start_time.strftime('%Y%m')
        if month_key not in month_groups:
            month_groups[month_key] = []
        month_groups[month_key].append(forcast_start_time_str)

        for hour in range(start_hour, end_hour + 1, forecast_interval):
            current_step = timedelta(hours=hour)
            forcast_time = forcast_start_time + current_step
            # logger.info(f'forcast_time: {forcast_time}')
            forcast_times.append(forcast_time.strftime("%Y-%m-%d %H:%M:%S"))

            # 格式化时间步长
            formatted_step = f"P{current_step.days}DT{current_step.seconds // 3600}H{(current_step.seconds // 60) % 60}M{current_step.seconds % 60}S"
            steps.append(formatted_step)
    # logger.info(f'forcast_times: {forcast_times}')
    # logger.info(f'forcast_start_times: {forcast_start_times}')

    # 获取实际查询的格点
    mapping_coords = approximate_coordinates(coords, nwp_meta['grid'])
    support_steps = json.loads(nwp_meta['steps'])
    valid_steps = [step for step in steps if step in support_steps]

    # 查询气象源数据
    result = query_mongo_multi_thread(workers, nwp_meta, month_groups, valid_steps, vars, mapping_coords)
    current = time.time()  # 记录查询结束时间
    query_time = current - query_start_time
    logger.info(f"查询耗时: {query_time:.2f} 秒")
    # 结果后处理
    result = handle_result(nwp_meta, forcast_start_times, valid_steps, src, start_time, period_interval, period, start_hour,
                           end_hour,
                           forecast_interval, mapping_coords, vars, result)
    all_end_time = time.time()  # 记录查询结束时间
    all_execution_time = all_end_time - query_start_time
    check_result(vars, forcast_start_times, valid_steps, result, nwp_meta)
    logger.info(f"总耗时: {all_execution_time:.2f} 秒")
    return result


def handle_result(nwp_meta: dict, forcast_start_times: list[str], valid_steps: list[str], src: str, start_time: str,
                  period_interval: int, period: int, start_hour: int, end_hour: int,
                  forecast_interval: int, mapping_coords: dict[tuple[float, float], Coordinate], vars: list[str],
                  result):
    if not result or len(result) == 0:
        return result
    df = pd.DataFrame(result)
    df = df.drop_duplicates()
    df = df.sort_values(by=['src', 'time', 'valid_time']).reset_index(drop=True)
    # 按经纬度分组
    grouped = df.groupby(['latitude', 'longitude'])
    # 构建结果列表
    result_list = []
    all_missing_forcast_times = set()
    for (lat, lon), group_df in grouped:
        group_df, missing_forcast_start_times =  find_missing_forcast_start_times(nwp_meta, lat, lon, group_df, forcast_start_times, valid_steps, vars)
        if missing_forcast_start_times:
            all_missing_forcast_times.update(missing_forcast_start_times)
        coor = mapping_coords.get((lat, lon))
        meta = {
            'src': src,
            'start_time': start_time,
            'period_interval': period_interval,
            'period': period,
            'start_hour': start_hour,
            'end_hour': end_hour,
            'forecast_interval': forecast_interval,
            'latitude': lat,
            'longitude': lon,
            'req_lat': coor.req_lat,
            'req_lon': coor.req_lon
        }
        # 构建 data 字典
        var_list_dict = {var: group_df[var].tolist() for var in vars}
        data = {
            # 'step': group_df['step'].tolist(),
            **var_list_dict
        }
        result_list.append({
            'meta': meta,
            'data': data
        })
    if all_missing_forcast_times:
        save_lost_forecast_times(all_missing_forcast_times, nwp_meta)
    return result_list


def check_result(vars, forcast_start_times, steps, result, nwp_meta):
    """
    :param vars: 所有气候变量
    :param forcast_start_times: 所有预报开始时间点
    :param steps 所有steps未去重
    :param result: 预报结果
    :param nwp_meta: 气象源元信息
    :return:
    """
    start_time = datetime.strptime(nwp_meta['start_time'], "%Y-%m-%d %H:%M:%S")
    end_time = datetime.strptime(nwp_meta['end_time'], "%Y-%m-%d %H:%M:%S")

    # 筛选出在指定时间范围内的所有起报时间
    # filtered_fc_start_times = [
    #     ft for ft in forcast_start_times
    #     if start_time <= datetime.strptime(ft, "%Y-%m-%d %H:%M:%S") <= end_time
    # ]
    # 去重步长
    unique_steps = list(set(steps))
    if not result or len(result) == 0:
        raise BizException(code=1, msg="查询结果为空")
    for index, item_result in enumerate(result):
        if not item_result['data']:
            raise BizException(code=2, msg="预报数据为空")
        for var in vars:
            # 检查是否存在该变量的预报数据
            if var not in result[index]['data']:
                raise BizException(code=3, msg=f"预报数据不存在于查询结果中")
            # 检查是否存在该预报时间点的预报数据
            expect_len = len(forcast_start_times) * len(unique_steps)
            real_len = len(result[index]['data'][var])
            if expect_len != real_len:
                raise BizException(code=4,
                                   msg=f"{var} 预报数据长度与预报时间点长度不一致, expect_len: {str(expect_len)}, real_len:{str(real_len)}")


def run_query(db: MongoClient, nwp_meta: dict, month_item: dict, forcast_times: list[str], steps: list[str],
              vars: list[str],
              mapping_coords: dict[tuple[float, float], Coordinate]):
    query_conditions = []

    # 获取近似经纬度
    rounded_coords = [
        {"req_lat": key[0], "req_lon": key[1]}
        for key in mapping_coords.keys()
    ]
    query_conditions.append({
        "src": nwp_meta['src'],
        "time": {"$in": forcast_times},
        "$or": [
            {"$and": [{"latitude": loc['req_lat']}, {"longitude": loc['req_lon']}]} for loc in rounded_coords
        ],
        "step": {"$in": steps},
        "$and": [{var: {"$exists": True}} for var in vars]
    })
    collection = get_collection(db, nwp_meta["prefix"], month_item)
    vars_dict = {key: 1 for key in vars}
    query = {"$and": query_conditions}
    if DEBUG:
        logger.info(f'query: {query_conditions}')
    # logger.info(f'query: {query_conditions}')
    query_results = collection.find(query, {
        "_id": 0,
        "src": 1,
        "time": 1,
        "valid_time": 1,
        "step": 1,
        "latitude": 1,
        "longitude": 1,
        **vars_dict
    }).sort({
        "time": 1,
        "step": 1
    })
    return list(query_results)


def query_mongo_multi_thread(workers, nwp_meta: dict, month_group: dict, steps: list[str], vars: list[str],
                             mapping_coords: dict[tuple[float, float], Coordinate]):
    # 并发查询
    all_results = []
    client = mongo_config.get_client()
    db = client[os.getenv("MONGO_DB", "auto_train")]
    with ThreadPoolExecutor(max_workers=workers) as executor:
        future_to_month = {
            executor.submit(run_query, db, nwp_meta, month_key, month_group[month_key], steps, vars,
                            mapping_coords): month_key
            for month_key in month_group
        }
        for future in concurrent.futures.as_completed(future_to_month):
            month_key = future_to_month[future]
            try:
                results = future.result()
                all_results.extend(results)
            except Exception as exc:
                print(f'Error for month {month_key}: {exc}')
    return all_results


def query_nwp_meta_info(src):
    """
    查询气象源元数据
    :param src: 气象源
    :return:
    """
    client = mongo_config.get_client()
    db = client[os.getenv("MONGO_DB", "auto_train")]
    meta_info = db["nwp_infos"].find_one({"src": src})
    if not meta_info:
        raise BizException(code=6, msg=f"未查询到{src}气象源元信息")
    meta_info['_id'] = str(meta_info['_id'])
    return meta_info


def get_collection(db, prefix, year_month):
    cname = f"{prefix}_{year_month}"
    if DEBUG:
        logger.info(f'collection name: {cname}')
    return db[cname]


def get_year_month(dt):
    year_month = dt.strftime("%Y%m")
    return year_month


def get_nearest_start_time(input_time_str, supported_hours):
    """
    获取与给定时刻最近（向前）的起报时间。

    参数:
    input_time_str (str): 输入的时间字符串，格式为 "YYMMDDHH"。
    supported_hours (list): 支持的小时时刻列表，如 ["00", "12"]。

    返回:
    str: 最近的起报时间字符串，格式为 "YYMMDDHH"。
    """
    # 解析输入的时间字符串
    input_time = datetime.strptime(input_time_str, "%y%m%d%H")

    # 找到最近的支持时刻
    nearest_hour = None
    min_diff = None
    for hour in supported_hours:
        # 构建当前支持时刻的时间
        test_time = datetime.strptime(input_time_str[:-2] + hour, "%y%m%d%H")

        # 计算差异
        diff = (input_time - test_time).total_seconds()

        # 只考虑过去的时间
        if diff >= 0 and (min_diff is None or diff < min_diff):
            nearest_hour = hour
            min_diff = diff

    if nearest_hour is None:
        raise ValueError("无法找到最近的支持时刻")

    # 返回最近的起报时间
    return input_time_str[:-2] + nearest_hour


def calculate_hours_between(start_time_str, end_time_str):
    """
    计算两个时间点之间的小时数。

    参数:
    start_time_str (str): 开始时间字符串，格式为 "YYMMDDHH"。
    end_time_str (str): 结束时间字符串，格式为 "YYMMDDHH"。

    返回:
    int: 两个时间点之间的小时数。
    """
    # 解析时间字符串为 datetime 对象
    start_time = datetime.strptime(start_time_str, "%y%m%d%H")
    end_time = datetime.strptime(end_time_str, "%y%m%d%H")

    # 计算时间差
    delta = end_time - start_time

    # 将时间差转换为小时数
    hours = int(delta.total_seconds() / 3600)

    return hours


def approximate_coordinates(coords, grid):
    """
    根据传入的分辨率对经纬度列表进行取舍，并存储在字典中。

    :param coords: 经纬度对象列表，每个对象包含 req_lat 和 req_lon 属性
    :param grid: 分辨率，单位为度
    :return: 存储近似后坐标的字典
    """
    if grid <= 0:
        raise ValueError("分辨率必须大于0")

    factor = 1 / grid
    mapping_coords = {}

    for coord in coords:
        approx_lon = round(coord.req_lon * factor) / factor
        approx_lat = round(coord.req_lat * factor) / factor
        key = (approx_lat, approx_lon)
        if key not in mapping_coords:
            mapping_coords[key] = coord

    return mapping_coords


from datetime import datetime


def get_real_start_time(nearest_start_time: str, latest_time: list, ndays: int) -> str:
    if not latest_time or len(latest_time) == 0:
        return nearest_start_time
    # 将nearest_start_time转换为datetime对象
    nearest_dt = datetime.strptime(nearest_start_time, '%y%m%d%H')
    # 将latest_time列表中的字符串转换为datetime对象
    latest_dts = [datetime.strptime(t, '%Y-%m-%d %H:%M:%S') for t in latest_time]
    # 找到latest_time中最接近nearest_start_time的时间
    closest_dt = min(latest_dts, key=lambda x: abs(x - nearest_dt))
    # 如果nearest_start_time比latest_time中的任何一个都靠前，返回nearest_start_time
    if all(nearest_dt < t for t in latest_dts):
        return nearest_start_time
    # 否则，将最接近的时间转换回yymmddhh格式
    closest_time_str = closest_dt.strftime('%y%m%d%H')
    return closest_time_str


def get_max_start_time(start_time: str, data_start_time: str) -> str:
    # 将start_time转换为datetime对象
    start_dt = datetime.strptime(start_time, '%y%m%d%H')
    # 将data_start_time转换为datetime对象
    data_start_dt = datetime.strptime(data_start_time, '%Y-%m-%d %H:%M:%S')
    # 比较两个时间，返回较大的时间
    max_dt = max(start_dt, data_start_dt)
    # 将较大的时间转换回yymmddhh格式
    max_time_str = max_dt.strftime('%y%m%d%H')
    return max_time_str

def get_max_start_time(start_time: str, data_start_time: str) -> str:
    # 将start_time转换为datetime对象
    start_dt = datetime.strptime(start_time, '%y%m%d%H')
    # 将data_start_time转换为datetime对象
    data_start_dt = datetime.strptime(data_start_time, '%Y-%m-%d %H:%M:%S')
    # 比较两个时间，返回较大的时间
    max_dt = max(start_dt, data_start_dt)
    # 将较大的时间转换回yymmddhh格式
    max_time_str = max_dt.strftime('%y%m%d%H')
    return max_time_str

def get_rule_start_time(start_time: str) -> str:
    # 将start_time转换为datetime对象
    start_dt = datetime.strptime(start_time, '%y%m%d%H')

    new_start_dt = start_dt - timedelta(hours=8)
    new_start_str = new_start_dt.strftime('%y%m%d%H')
    return new_start_str

def find_missing_forcast_start_times(nwp_meta: dict, latitude: float, longitude: float, df: pd.DataFrame,
                                     forcast_start_times: list[str], valid_steps: list[str],
                                     vars: list[str]) -> pd.DataFrame:
    missing_forcast_start_times = set()
    missing_data = []

    for time in forcast_start_times:
        # 过滤出对应时间的数据
        filtered_df = df[df['time'] == time]
        # 获取当前时间的所有step
        current_steps = set(filtered_df['step'])
        # 找到缺失的step
        missing_steps_for_time = set(valid_steps) - current_steps

        # 如果有缺失的step，记录下来
        if missing_steps_for_time:
            missing_forcast_start_times.add(time)

            # 生成缺失数据的行
            for step in missing_steps_for_time:
                row = {
                    'time': time,
                    'step': step,
                    'valid_time': calculate_valid_time(time, step),
                    'src': nwp_meta['src'],
                    'latitude': latitude,
                    'longitude': longitude
                }
                for var in vars:
                    row[var] = np.nan
                missing_data.append(row)

    # 将缺失数据添加到 DataFrame 中
    if missing_data:
        missing_df = pd.DataFrame(missing_data)
        df = pd.concat([df, missing_df], ignore_index=True)
        df = df.sort_values(by=['src', 'time', 'valid_time']).reset_index(drop=True)
    return df, missing_forcast_start_times


def save_lost_forecast_times(missing_forcast_start_times: set, nwp_meta: dict):
    """
    更新缺失的预报时间次
    """
    client = mongo_config.get_client()
    db = client[os.getenv("MONGO_DB", "auto_train")]
    lost_forecast_times = nwp_meta.get('lost_forecast_times', [])
    merged_forecast_times = list(set(lost_forecast_times + list(missing_forcast_start_times)))
    result = db["nwp_infos"].update_one(
        {'_id': ObjectId(nwp_meta['_id'])},
        {'$set': {'lost_forecast_times': merged_forecast_times}}
    )



def parse_step(step: str) -> relativedelta:
    """
    解析step字符串，返回一个relativedelta对象。
    step格式为PnDTnHnMnS，例如P2DT2H0M0S。
    """
    # 去掉开头的P
    step = step[1:]

    # 定义正则表达式来匹配各个部分
    pattern = r'(?:(\d+)D)?T?(?:(\d+)H)?(?:(\d+)M)?(?:(\d+)S)?'
    match = re.match(pattern, step)

    if not match:
        raise ValueError(f"Invalid step format: {step}")

    groups = match.groups()

    # 解析各部分
    days = int(groups[0]) if groups[0] else 0
    hours = int(groups[1]) if groups[1] else 0
    minutes = int(groups[2]) if groups[2] else 0
    seconds = int(groups[3]) if groups[3] else 0

    return relativedelta(days=days, hours=hours, minutes=minutes, seconds=seconds)


def calculate_valid_time(time: str, step: str) -> str:
    """
    计算valid_time，给定time和step。
    time格式为YYYY-MM-DD HH:MM:SS，例如2024-08-01 00:00:00。
    step格式为PnDTnHnMnS，例如P2DT2H0M0S。
    返回valid_time，格式为YYYY-MM-DD HH:MM:SS。
    """
    time_dt = datetime.strptime(time, '%Y-%m-%d %H:%M:%S')
    step_delta = parse_step(step)
    valid_time_dt = time_dt + step_delta
    return valid_time_dt.strftime('%Y-%m-%d %H:%M:%S')




import json
import os
import time

import pandas as pd
import requests
from loguru import logger
from typing import Union
from sais.autotrain.dataservice.auth.auth_info import EnvVarCredentialsProvider
from sais.autotrain.dataservice.client.mongo_client import query_nwp_mongo, query_nwp_meta_info, query_nwp_mongo_biz
from sais.autotrain.dataservice.config.const import ENDPOINT
from sais.autotrain.dataservice.model.data_model import Task, StatusEnum, QueryNWPRequest, QueryNWPBizRequest, \
    QueryStationRequest, \
    QueryObservationRequest, BaseRsp, BizResponse, QueryAreaRequest
from sais.autotrain.dataservice.types.biz_exception import BizException
from sais.autotrain.dataservice.config.mongo_config import mongo_config


class QueryNWPHandler(object):
    def __init__(self, endpoint=os.getenv("ENDPOINT", ENDPOINT), auth_provider: EnvVarCredentialsProvider = None,
                 mongo_host=None, mongo_port=None, mongo_user=None, mongo_password=None):
        self.endpoint = endpoint
        self.auth_provider = auth_provider
        if mongo_host and mongo_port and mongo_user and mongo_password:
            mongo_config.set_config(host=mongo_host, port=mongo_port, user=mongo_user, password=mongo_password)

    def submit_query_task(self, query: QueryNWPRequest):
        """提交查询任务并返回任务ID"""
        url = f"{self.endpoint}/api/v1/query"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        response = requests.post(url, json=query.model_dump(by_alias=True, exclude_none=True), headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            logger.info(f"Query task submitted successfully. Task ID: {query_result_json.get('data').get('task_id')}")
            return query_result_json.get('data').get('task_id')
        else:
            logger.error(f"Failed to submit query task. Response: {query_result_json}")
            raise ValueError(f"Failed to submit query task.")

    def query_task_status(self, task_id):
        """通过任务ID查询任务状态"""
        url = f"{self.endpoint}/api/v1/query/{task_id}"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}"
        }
        response = requests.get(url, headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            logger.info(f"Query task status: {query_result_json.get('data')}")
            return Task(**query_result_json.get('data'))
        else:
            logger.error(f"Failed to query task status. Response: {query_result_json}")
            raise ValueError(f"Failed to query task status.{query_result_json.get('msg')}")

    def wait_for_task_completion(self, task_id, poll_interval=5):
        """
        轮询查询任务状态直到任务完成
        状态：
            - PENDING: 等待执行
            - RUNNING: 执行中
            - FINISHED: 完成（已成功生成json文件）
            - FAILED: 失败
        """
        while True:
            task = self.query_task_status(task_id)
            if task.status == StatusEnum.FINISHED:
                if os.path.exists(task.file_path):
                    with open(task.file_path, 'r') as file:
                        return json.load(file)
                else:
                    return None
            elif task.status == StatusEnum.FAILED:
                raise Exception(f"Query task failed. Task ID: {task.task_id}")
            time.sleep(poll_interval)  # 轮询间隔

    def execute_query_mongo(self, query_params: Union[QueryNWPRequest, QueryNWPBizRequest]):
        if isinstance(query_params, QueryNWPRequest):
            return query_nwp_mongo(query_params.src, query_params.start_time, query_params.period_interval,
                                   query_params.period,
                                   query_params.start_hour,
                                   query_params.end_hour,
                                   query_params.forecast_interval,
                                   query_params.coords,
                                   query_params.vars,
                                   query_params.workers)
        elif isinstance(query_params, QueryNWPBizRequest):
            return query_nwp_mongo_biz(query_params.src, query_params.start_time, query_params.ndays,
                                       query_params.hours,
                                       query_params.coords,
                                       query_params.vars,
                                       query_params.workers)
        else:
            raise TypeError("Unsupported query parameters type")

    def execute_query(self, query_params: Union[QueryNWPRequest, QueryNWPBizRequest]):
        if query_params.engine == "mongo":
            try:
                result = BizResponse(data=self.execute_query_mongo(query_params))
            except BizException as biz_err:
                result = BizResponse(code=biz_err.code, msg=biz_err.msg)
            except Exception as e:
                result = BizResponse(code=1, msg=str(e))
            return result
        else:
            """封装的查询功能函数，提交查询任务并轮询直到返回任务状态结果"""
            task_id = self.submit_query_task(query_params)
            print(f"Query task submitted with task ID: {task_id}")
            return self.wait_for_task_completion(task_id)

    def query_nwp_meta(self, src):
        """获取nwp数据源的元数据"""
        try:
            result = BizResponse(data=query_nwp_meta_info(src))
        except BizException as biz_err:
            result = BizResponse(code=biz_err.code, msg=biz_err.msg)
        except Exception as e:
            result = BizResponse(code=1, msg=str(e))
        return result


class QueryStationRelatedHandler(object):
    def __init__(self, endpoint=ENDPOINT, auth_provider: EnvVarCredentialsProvider = None):
        self.endpoint = endpoint
        self.auth_provider = auth_provider

    def execute_query_area(self, query_params):
        """场站查询"""
        try:
            area_data = self.submit_query_area(query_params)
            result = BizResponse(code=0, data=area_data)
        except BizException as biz_err:
            result = BizResponse(code=biz_err.code, msg=biz_err.msg)
        except Exception as e:
            result = BizResponse(code=1, msg=str(e))
        return result

    def execute_query_station(self, query_params):
        """场站查询"""
        try:
            station_data = self.submit_query_station(query_params)
            result = BizResponse(code=0, data=station_data)
        except BizException as biz_err:
            result = BizResponse(code=biz_err.code, msg=biz_err.msg)
        except Exception as e:
            result = BizResponse(code=1, msg=str(e))
        return result

    def execute_query_observation(self, query_params):
        """观测查询"""
        try:
            observation_data = self.submit_query_observation(query_params)
            result = BizResponse(code=0, data=observation_data)
        except BizException as biz_err:
            result = BizResponse(code=biz_err.code, msg=biz_err.msg)
        except Exception as e:
            result = BizResponse(code=1, msg=str(e))
        return result

    def execute_label_observation(self, df: pd.DataFrame):
        """标记观测查询"""
        try:
            update_result = self.submit_label_observation(df)
            result = BizResponse(code=0, data=update_result)
        except BizException as biz_err:
            result = BizResponse(code=biz_err.code, msg=biz_err.msg)
        except Exception as e:
            result = BizResponse(code=1, msg=str(e))
        return result

    def submit_query_observation(self, query: QueryObservationRequest):
        """提交观测查询"""
        url = f"{self.endpoint}/api/v1/observation/query"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        response = requests.post(url, json=query.model_dump(by_alias=True, exclude_none=True), headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            return BaseRsp(**query_result_json.get('data'))
        else:
            logger.error(f"Failed to submit query observation. Response: {query_result_json}")
            raise ValueError(f"Failed to submit query observation.{query_result_json.get('msg')}")

    def submit_query_area(self, query: QueryAreaRequest):
        """提交省份或区域查询"""
        url = f"{self.endpoint}/api/v1/area/query"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        response = requests.post(url, json=query.model_dump(by_alias=True, exclude_none=True), headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            return BaseRsp(**query_result_json.get('data'))
        else:
            logger.error(f"Failed to submit query area. Response: {query_result_json}")
            raise ValueError(f"Failed to submit area station.{query_result_json.get('msg')}")

    def submit_query_station(self, query: QueryStationRequest):
        """提交场站查询"""
        url = f"{self.endpoint}/api/v1/station/query"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        response = requests.post(url, json=query.model_dump(by_alias=True, exclude_none=True), headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            return BaseRsp(**query_result_json.get('data'))
        else:
            logger.error(f"Failed to submit query station. Response: {query_result_json}")
            raise ValueError(f"Failed to submit query station.{query_result_json.get('msg')}")

    def submit_label_observation(self, df: pd.DataFrame):
        """提交标记观测数据"""
        url = f"{self.endpoint}/api/v1/observation/label"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        data = {
            "observations": df.to_dict(orient='records')
        }
        response = requests.post(url, json=data, headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            return True
        else:
            logger.error(f"Failed to submit label observation. Response: {query_result_json}")
            return False



import os


class CustomEnv:
    def __init__(self):
        pass

    @classmethod
    def get_redis_host(cls) -> str:
        return "localhost" if os.getenv("REDIS_HOST") is None else os.getenv("REDIS_HOST")

    @classmethod
    def get_redis_password(cls) -> str:
        return "default_password" if os.getenv("REDIS_PASSWORD") is None else os.getenv("REDIS_PASSWORD")

    @classmethod
    def get_redis_port(cls) -> int:
        return 6379 if os.getenv("REDIS_PORT") is None else os.getenv("REDIS_PORT")

    @classmethod
    def get_redis_db(cls) -> int:
        return 0 if os.getenv("REDIS_DB") is None else os.getenv("REDIS_DB")

    @classmethod
    def get_server_port(cls) -> str:
        return os.getenv("SERVER_PORT") if os.getenv("SERVER_PORT") else "10019"